﻿using ProjectAircraftCarrier.Entities;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ProjectAircraftCarrier.Drawnings;

/// <summary>
/// Класс отвечающий за прорисовку и перемещение базового объекта-сущности
/// </summary>
public class DrawningWarship
{
    /// <summary>
    /// Класс-сущность
    /// </summary>
    public EntityWarship? EntityWarship { get; protected set; }

    /// <summary>
    /// Ширина окна
    /// </summary>
    private int? _pictureWidth;

    /// <summary>
    /// Высота окна
    /// </summary>
    private int? _pictureHeight;

    /// <summary>
    /// Левая координата прорисовки военного корабля
    /// </summary>
    protected int? _startPosX;

    /// <summary>
    /// Верхняя координата отрисовки военного корабля
    /// </summary>
    protected int? _startPosY;

    /// <summary>
    /// Ширина прорисовки военного корабля
    /// </summary>
    private readonly int _drawningWarshipWidth = 145;

    /// <summary>
    /// Высота прорисовки военного корабля
    /// </summary>
    private readonly int _drawningWarshipHeight = 50;

    /// <summary>
	/// Координата X объекта
	/// </summary>
	public int? GetPosX => _startPosX;

    /// <summary>
    /// Координата Y объекта
    /// </summary>
    public int? GetPosY => _startPosY;

    /// <summary>
    /// Ширина объекта
    /// </summary>
    public int GetWidth => _drawningWarshipWidth;

    /// <summary>
    /// Высота объекта
    /// </summary>
    public int GetHeight => _drawningWarshipHeight;

    /// <summary>
    /// Пустой конструктор
    /// </summary>
    private DrawningWarship()
    {
        _pictureWidth = null;
        _pictureHeight = null;
        _startPosX = null;
        _startPosY = null;
    }

    /// <summary>
    /// Конструктор
    /// </summary>
    /// <param name="speed">Скорость</param>
    /// <param name="weight">Вес</param>
    /// <param name="bodyColor">Основной цвет</param>
    public DrawningWarship(int speed, double weight, Color bodyColor) : this()
    {
        EntityWarship = new EntityWarship(speed, weight, bodyColor);
    }

    /// <summary>
    /// Конструктор для наследников
    /// </summary>
    /// <param name="drawningWarshipWidth">Ширина прорисовки военного корабля</param>
    /// <param name="drawningWarshipHeight">Высота прорисовки военного корабля</param>
    protected DrawningWarship(int drawningWarshipWidth, int drawningWarshipHeight) : this()
    {
        _drawningWarshipWidth = drawningWarshipWidth;
        _drawningWarshipHeight = drawningWarshipHeight;
    }

    /// <summary>
    /// Установка границ поля
    /// </summary>
    /// <param name="width">Ширина поля</param>
    /// <param name="height">Высота поля</param>
    /// <returns>true - границы заданы, false - проверка не пройдена, нельзя разместить объект в этих размерах</returns>
    public bool SetPictureSize(int width, int height)
    {
        if (width < _drawningWarshipWidth || height < _drawningWarshipHeight)
        {
            return false;
        }

        _pictureWidth = width;
        _pictureHeight = height;
        if (_startPosX != null || _startPosY != null)
        {
            if (_startPosX + _drawningWarshipWidth > _pictureWidth)
            {
                _startPosX = -_drawningWarshipWidth + _pictureWidth;
            }
            else if (_startPosX < 0)
            {
                _startPosX = 0;
            }
            if (_startPosY + _drawningWarshipHeight > _pictureHeight)
            {
                _startPosY = -_drawningWarshipHeight + _pictureHeight;
            }
            else if (_startPosY < 0)
            {
                _startPosY = 0;
            }
        }
        return true;

    }

    /// <summary>
    /// Установка позиции
    /// </summary>
    /// <param name="x">Координата X</param>
    /// <param name="y">Координата Y</param>
    public void SetPosition(int x, int y)
    {
        if (!_pictureHeight.HasValue || !_pictureWidth.HasValue)
        {
            return;
        }

        if (x < 0 || x + _drawningWarshipWidth > _pictureWidth)
        {
            x = 0;
        }

        if (y < 0 || y + _drawningWarshipHeight > _pictureHeight)
        {
            y = 0;
        }

        _startPosX = x;
        _startPosY = y;
    }

    /// <summary>
    /// Измнение направления перемещения
    /// </summary>
    /// <param name="direction">Направление</param>
    /// <returns>true - перемещение выполнено, false - перемещение невозможно</returns>
    public bool MoveTransport(DirectionType direction)
    {
        if (EntityWarship == null || !_startPosX.HasValue || !_startPosY.HasValue || !_pictureWidth.HasValue || !_pictureHeight.HasValue)
        {
            return false;
        }

        switch (direction)
        {
            //влево
            case DirectionType.Left:
                if (_startPosX.Value - EntityWarship.Step >= 0)
                {
                    _startPosX -= (int)EntityWarship.Step;
                }
                return true;
            //вверх
            case DirectionType.Up:
                if (_startPosY.Value - EntityWarship.Step >= 0)
                {
                    _startPosY -= (int)EntityWarship.Step;
                }
                return true;
            //вправо
            case DirectionType.Right:
                if (_startPosX.Value + _drawningWarshipWidth + EntityWarship.Step <= _pictureWidth)
                {
                    _startPosX += (int)EntityWarship.Step;
                }
                return true;
            //вниз
            case DirectionType.Down:
                if (_startPosY.Value + _drawningWarshipHeight + EntityWarship.Step <= _pictureHeight)
                {
                    _startPosY += (int)EntityWarship.Step;
                }
                return true;
            default:
                return false;
        }
    }

    /// <summary>
    /// Прорисовка объекта
    /// </summary>
    /// <param name="g"></param>
    public virtual void DrawTransport(Graphics g)
    {
        if (EntityWarship == null || !_startPosX.HasValue || !_startPosY.HasValue)
        {
            return;
        }

        Pen pen = new(Color.Black);

        //границы военного корабля
        Point[] FramePoints = {
            new Point(_startPosX.Value + 5, _startPosY.Value),
            new Point(_startPosX.Value + 105, _startPosY.Value),
            new Point(_startPosX.Value + 145, _startPosY.Value + 25),
            new Point(_startPosX.Value + 105, _startPosY.Value + 50),
            new Point(_startPosX.Value + 5, _startPosY.Value + 50),
        };
        g.DrawPolygon(pen, FramePoints);
        g.DrawRectangle(pen, _startPosX.Value, _startPosY.Value + 5, 5, 18);
        g.DrawRectangle(pen, _startPosX.Value, _startPosY.Value + 27, 5, 18);

        //корпус военного корабля
        Brush br = new SolidBrush(EntityWarship.BodyColor);
        g.FillPolygon(br, FramePoints);

        //отсеки военного корабля
        Brush brGray = new SolidBrush(Color.Gray);
        g.FillRectangle(brGray, _startPosX.Value + 80, _startPosY.Value + 10, 15, 30);
        g.FillRectangle(brGray, _startPosX.Value + 60, _startPosY.Value + 20, 20, 10);
        g.FillEllipse(brGray, _startPosX.Value + 100, _startPosY.Value + 14, 20, 20);

        //выделяем рамкой отсеки военного корабля
        g.DrawRectangle(pen, _startPosX.Value + 80, _startPosY.Value + 10, 15, 30);
        g.DrawRectangle(pen, _startPosX.Value + 60, _startPosY.Value + 20, 20, 10);
        g.DrawEllipse(pen, _startPosX.Value + 100, _startPosY.Value + 14, 20, 20);

        //двигатели военного корабля
        Brush brBlack = new SolidBrush(Color.Black);
        g.FillRectangle(brBlack, _startPosX.Value, _startPosY.Value + 5, 5, 18);
        g.FillRectangle(brBlack, _startPosX.Value, _startPosY.Value + 27, 5, 18);
    }
}
